import sqlalchemy
from DataModel import db, session
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session
import DataModel.models as models
from pydantic import BaseModel
import bcrypt
from fastapi import FastAPI,File, UploadFile,APIRouter,Depends
from typing import List
from fastapi.responses import JSONResponse
import aiofiles
import random
import string
import time
from typing import Dict
from jose import JWTError, jwt
from fastapi import Request, HTTPException,status
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
import logging,os
from email_validator import validate_email, EmailNotValidError
from Endpoints.image_validation import image_validation

logging.basicConfig(filename="DataModel//logfile.txt", level=logging.DEBUG,
                   format="%(asctime)s :%(levelname)s :%(message)s %(lineno)d", filemode="w")
logger = logging.getLogger()


router = APIRouter()
app = FastAPI()

JWT_SECRET = "967586b953867b3db8dc33ed44452e27"
JWT_ALGORITHM = "HS256"

class UserBase(BaseModel):
    email: str

class UserCreate(UserBase):
    password: str
    emp_name: str
    emp_code: str
    mobile: str

class User(UserCreate):
    id: int

    # ctgresults: list['ctgresults'] = []
    # date_reg: _dt
    class Config:
        orm_mode = True

def token_response(token: str):
    return {
        "access_token": token
    }

def signJWT(email: str) -> Dict[str, str]:
    payload = {
        "email": email,
        "expires": time.time() + 600
    }
    token = jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM)
    return token_response(token)


def decodeJWT(token: str) -> dict:
    try:
        decoded_token = jwt.decode(token, JWT_SECRET, algorithms=[JWT_ALGORITHM])
        return decoded_token if decoded_token["expires"] >= time.time() else None
    except:
        return {}

class JWTBearer(HTTPBearer):
    def __init__(self, auto_error: bool = True):
        super(JWTBearer, self).__init__(auto_error=auto_error)

    async def __call__(self, request: Request):
        credentials: HTTPAuthorizationCredentials = await super(JWTBearer, self).__call__(request)
        if credentials:
            if not credentials.scheme == "Bearer":
                raise HTTPException(status_code=403, detail="Invalid authentication scheme.")
            if not self.verify_jwt(credentials.credentials):
                raise HTTPException(status_code=403, detail="Invalid token or expired token.")
            return credentials.credentials
        else:
            raise HTTPException(status_code=403, detail="Invalid authorization code.")

    def verify_jwt(self, jwtoken: str) -> bool:
        isTokenValid: bool = False
        try:
            payload = decodeJWT(jwtoken)
        except:
            payload = None
        if payload:
            isTokenValid = True
        return isTokenValid

@router.post("/UserRegistration/",response_model=User)
async def create_user(user: UserCreate):
    u = sqlalchemy.select(models.User).where(models.User.email == user.email and models.User.emp_code == user.emp_code)
    email_ex = db.execute(u).fetchone()
    if email_ex:
        raise HTTPException(status_code=400, detail=f"User already registered with the provided email or emp_code")
    try:
        db_user = models.User(email=validate_email(user.email).email,
                              password=bcrypt.hashpw(user.password.encode('utf-8'), bcrypt.gensalt(10)),
                              mobile=user.mobile,
                              emp_name=user.emp_name,
                              emp_code=user.emp_code)
        session.add(db_user)
        session.commit()
        session.refresh(db_user)
        return db_user

    except SQLAlchemyError as e:
        error = str(e)
        logger.error(error)
        return {"error_del": error}
    except EmailNotValidError as e:
        # email is not valid, exception message is human-readable
        logger.exception(str(e))
        return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST,
            content={'message': str(e)})


@router.post("/UserLogin/")
async def read_item(email: str,password: str):
    try:
        login = sqlalchemy.select(models.User).where(models.User.email == email)
        user_details = db.execute(login).fetchone()
        if bcrypt.checkpw(password.encode('utf-8'), user_details.password):
            return {"jwt_token": signJWT(email), "id": user_details.id, "emp_code": user_details.emp_code,
                    "emp_name": user_details.emp_name}
        else:
             return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST, content={"message":"Incorrect login credentials"}
        )
    except SQLAlchemyError as e:
        error = str(e)
        logger.error(error)
        return {"error_add": error}
    except Exception as e:
        logger.exception(str(e))
        return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST, content={"message": f"Incorrect login credentials"}
        )

@router.post("/ImageanAlysis", dependencies=[Depends(JWTBearer())])
async def image_analysis(user_id: str,files: List[UploadFile] = File(...)):
    destination_file_path =''
    for file in files:
        if file.filename.endswith((".pdf", ".jpg", ".png")):
            path_cwd = os.getcwd()
            destination_file_path = path_cwd + "\\Endpoints\\SRMC_INPUT_REPORTS\\" + file.filename
            async with aiofiles.open(destination_file_path, 'wb') as out_file:
                while content := await file.read(1024):  # async read file chunk
                    await out_file.write(content)  # async write file chunk
            continue

    from Endpoints.Ctgireport import CtgiReport
    if "InValid" not in image_validation(destination_file_path):
        text = CtgiReport(user_id,destination_file_path).Pdf_Report()
        file = open(destination_file_path,"rb")
        file.close()
        os.remove(destination_file_path)
        return text
    else:
        return {"Image anlysis_status": "Invalid image"}
@router.delete('/delete_record')
async def delete_record_by_id(user_id: str):
    u = sqlalchemy.select(models.Ctgiresults).where(models.Ctgiresults.user_id == user_id)
    id_ex = db.execute(u).fetchone()
    if not id_ex:
        raise HTTPException(status_code=404, detail=f"No record found to delete")
    try:
        u = sqlalchemy.delete(models.Ctgiresults).where(models.Ctgiresults.user_id == user_id)
        session.execute(u)
        session.commit()
    except SQLAlchemyError as e:
        error = str(e)
        logger.error(error)
        return {"error_del": error}
    except Exception as e:
        logger.exception(e)
        raise HTTPException(status_code=400, detail=f"Unable to delete: {e}")
    return {"delete_status": "success"}


@router.get("/search")
async def read_item(User_id: str,search_parameter:str,page_no:int = 1,page_size : int = 2):
    login = sqlalchemy.select(models.Ctgiresults).where(models.Ctgiresults.user_id == User_id and
                    (models.Ctgiresults.patientName == search_parameter or
                     models.Ctgiresults.result == search_parameter or
                     models.Ctgiresults.patientId == search_parameter or
                     models.Ctgiresults.timeStamp == search_parameter))
    user_details = db.execute(login).fetchall()
    data_length = len(user_details)
    start = (page_no-1)*page_size
    end = start+page_size
    response= {
        "data":user_details[start:end],
        "total":data_length,
        "count":page_size,
        "pagination":{}
    }
    if end>=data_length:
        response["pagination"]["next"]= None
        if page_no>1:
            response["pagination"]["previous"]=f"/posts/page_num = {page_no-1} & page_size = {page_size}"
        else:
            response["pagination"]["previous"] = None

    else:
        if page_no > 1:
            response["pagination"]["previous"] = f"/posts/page_num = {page_no - 1} & page_size = {page_size}"
        else:
            response["pagination"]["previous"] = None

        response["pagination"]["next"] = f"/posts/page_num = {page_no + 1} & page_size = {page_size}"

    return response
